\\ Copyright 2014, 2015, 2016, 2017, 2018, 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ THIS IS A GENERATED FILE
\\ Extracted from rationals.tex by gp-inline.
\\
\\ These are Pari/GP functions and definitions used in rationals.tex for
\\ various self-tests and formula checking there.  Names and styles are
\\ highly specific to the calculations there, and many things will change or
\\ disappear, etc, but some of the functions might be useful for
\\ experimenting or testing.


/* ------------------------------------------------------------------------- */
/* gp-inline test boilerplate begin */
gp_inline__location = "";
gp_inline__good = 0;
gp_inline__bad = 0;
gp_inline__check(location,bool) =
{
  gp_inline__location = location;
  check(bool);
}
/* this is thinking about a check() for use in loops or "GP-Inline ..."
   testing, but it's not settled so don't rely on it */
check(bool) =
{
  /* use "===" so that a vector like [1] or float 1.0 not reckoned success */
  if(bool===1, gp_inline__good++,
     bool===0, gp_inline__bad++;
               print(gp_inline__location": gp-inline fail"),
     gp_inline__bad++;
     print(gp_inline__location": gp-inline expected result 0 or 1, got ",
           bool));
}
/* gp-inline test boilerplate end */
/* ------------------------------------------------------------------------- */

gp_inline__location="../rationals.tex:221";
 default(strictargs,1);

gp_inline__location="../rationals.tex:225";
 \\ my http://user42.tuxfamily.org/pari-geom/index.html
 iferr(read("tools/geom.gp"), e, \
    read("../tools/geom.gp"));
 iferr(read("tools/geom-inertia.gp"), e, \
    read("../tools/geom-inertia.gp"));
 iferr(read("tools/geom-wip.gp"), e, \
    read("../tools/geom-wip.gp"));

gp_inline__location="../rationals.tex:233";
 \\ my http://user42.tuxfamily.org/pari-memoize/index.html
 iferr(read("tools/memoize.gp"),e, \
    read("../tools/memoize.gp"));

gp_inline__location="../rationals.tex:237";
 iferr(read("tools/rationals-oeis-samples.gp"),e, \
    read("../tools/rationals-oeis-samples.gp"));

gp_inline__location="../rationals.tex:240";
 nearly_equal_epsilon = 1e-15;
 nearly_equal(x,y, epsilon=nearly_equal_epsilon) = {
   \\ print(type(x)," ",type(y)," ",x," ",y);
   my(d=x-y); \\ ensure compatible types
   if(type(x)=="t_MAT",x=Vec(x);y=Vec(y));
   if(type(x)=="t_VEC" || type(x)=="t_COL",
        sum(i=1,max(#x,#y), !nearly_equal(x[i],y[i],epsilon)) == 0,
      type(x)=="t_QUAD" || type(x)=="t_COMPLEX",
        nearly_equal(abs(x),abs(y),epsilon),
      abs(d) < epsilon);
 }

gp_inline__location="../rationals.tex:252";
 sqrt5 = geom_sqrt_as_quad(5);
 sqrt17 = geom_sqrt_as_quad(17);
 phi = (1+sqrt5)/2;

gp_inline__location="../rationals.tex:261";
 F(n)=fibonacci(n);

gp_inline__location="../rationals.tex:263";
 \\ Lucas numbers A000032
 V(n) = F(n+1) + F(n-1);
 A000032(n) = V(n);

gp_inline__location="../rationals.tex:278";
 F_phi(n) = phi^n / sqrt5;

gp_inline__location="../rationals.tex:281";
 \\ return n with the bits below the most significant reversed
 \\ A059893
 n_bit_reversal(n) = {
   my(v=binary(n));
   if(#v>=2,v=concat(v[1..1],Vecrev(v[2..#v])));
   fromdigits(v,2);
 }
 A059893(n) = n_bit_reversal(n);

gp_inline__location="../rationals.tex:294";
 \\ n is an integer with decimal digits representing bits.
 \\ Return the value of those bits.
 from_binary(n) = fromdigits(digits(n),2);

gp_inline__location="../rationals.tex:300";
 \\ n is an integer.  Return an integer whose decimal digits
 \\ are the bits of n.
 to_binary(n)=fromdigits(binary(n));

gp_inline__location="../rationals.tex:306";
 \\ n is an integer.  Return its bits as a vector high to low
 \\ padded out with high 0s to make k many bits.
 \\ n must fit within k bits, ie. 0 <= n < 2^k.
 binary_padded(n,k) = {
   my(v = binary(n));
   if(#v>k, error("binary_padded() n=",n," is bigger than k=",k," bits"));
   concat(vector(max(0,k-#v)),v);
 }

gp_inline__location="../rationals.tex:318";
 bitlength(n) = if(n==0,0, logint(n,2)+1);

gp_inline__location="../rationals.tex:322";
 \\ return 1 or 0 according as x is an integer or not
 is_integer(x) = (x==floor(x));

gp_inline__location="../rationals.tex:325";
 \\ Return 1 if x is an integer power of 2, so 2^0, 2^1, etc.
 \\ Return 0 if not.
 is_power_of_2(x) = is_integer(x) && x>=1 && x==2^valuation(x,2);

gp_inline__location="../rationals.tex:336";
 \\ v is a vector of fractional decimal digits high to low.
 \\ Eg. [1,2,3] is 123/1000 = 0.123
 \\ Return them as a rational like 12981/10^5.
 \\ If v is longer than the current floating point
 \\ precision() then use only that much of v.
 \\ "point" is how many digits before the decimal point.
 \\ "add" is an amount to add at the low end of the digits.
 fromdigits_to_precision(v,point=0, add=0) = {
   my(p = precision(1.0) - 4);
   if(#v>p, v = v[1..p]);
   (fromdigits(v)+add) / 10^(#v-point);
 }

gp_inline__location="../rationals.tex:352";
 \\ Return a vector of the first n coefficients of
 \\ generating function g, starting from term x^0.
 \\ g is normally a polynomial fraction (type t_RFRAC)
 \\ but a plain polynomial is accepted too and returns
 \\ its lowest n coefficients.
 gf_terms(g,n) = \
   my(x = variable(g), \
      zeros = min(n,valuation(g,x)), \
      v = Vec(g + O(x^n))); \
   if(zeros>=0, concat(vector(zeros,i,0), v), \
                v[-zeros+1 .. #v]);

gp_inline__location="../rationals.tex:371";
 gf_term(g,n) = gf_terms(g,n+1)[n+1];

gp_inline__location="../rationals.tex:373";
 points_to_complex(v) = apply((m)->vec_to_p(m) + I*vec_to_q(m), v);

gp_inline__location="../rationals.tex:375";
 \\ v is a vector of continued fraction terms, as per contfrac().
 \\ Return the fraction it represents.
 \\ Eg. v=[9,8,7,6] gives 9 + 1/(8 + 1/(7 + 1/6))
 contfrac_to_rational(v) = {
   \\ contfracpnqn() gives matrix [ p[n], p[n-1]; 
   \\                               q[n], q[n-1] ]
   \\ where p[n]/q[n] is the last partial quotient,
   \\ and p[n-1],q[n-1] is the second last (unused here)
   my(m=contfracpnqn(v));
   m[1,1]/m[2,1];
 }

gp_inline__location="../rationals.tex:391";
 \\ select_first_n(f,n) returns a vector of length n which are the
 \\ indices i for which function f(i) is true.
 \\ Indices checked start from 0 so i=0,1,2,...
 select_first_n(f,n) = {
   my(l=List([]), i=0);
   while(#l<n, if(f(i),listput(l,i)); i++);
   Vec(l);
 }

gp_inline__location="../rationals.tex:400";
 \\ Remainder from n divide d, with quotient rounded up
 \\ so q = ceil(n/d) and n = q*d + r
 mod_ceil(n,d) = my(q=ceil(n/d)); n - q*d;

gp_inline__location="../rationals.tex:404";
 \\ "table" is a matrix where each row is state transitions,
 \\ so new_state = table[state,digit].
 \\ Return a vector where v[state] = generating function for how
 \\ many digit strings are in state after k steps.
 \\ "initial" is the starting state.  The k=0 terms of the gfs are
 \\ 1 in v[initial] and 0 everywhere else.
 state_table_to_gfs(table,initial=1) = {
   my(m=state_table_to_matrix(table),
      l=matsize(m)[1],
      v=vectorv(l));
   v[initial]=1; \\ and all others 0 (index error on bad "initial")
   Vec(matsolve(matid(l)-m*'x, v));
 }

gp_inline__location="../rationals.tex:418";
 state_table_to_matrix(table) = {
   my(l=matsize(table)[1], w=matsize(table)[2], m=matrix(l,l));
   for(from=1,l,
     for(digit=1,w,
       m[table[from,digit],from]++));
   m;
 }

gp_inline__location="../rationals.tex:430";
 vec_to_p(v)=v[1,1];     /* extract p from vector */
 vec_to_q(v)=v[2,1];     /* extract q from vector */
 vec_to_p_minus_q(v) = vec_to_p(v) -   vec_to_q(v); \\ p-q
 vec_to_p_plus_q(v)  = vec_to_p(v) +   vec_to_q(v); \\ p+q
 vec_to_p_plus_2q(v) = vec_to_p(v) + 2*vec_to_q(v); \\ p+2q
 vec_to_frac(v) = vec_to_p(v) / vec_to_q(v); \\ p/q fraction
 point_transpose(v) = [vec_to_q(v); vec_to_p(v)];

gp_inline__location="../rationals.tex:438";
 vector_mean(v) = vecsum(v)/#v;
 vector_modulo(v,i) = v[(i% #v)+1];

gp_inline__location="../rationals.tex:442";
 a1 = {[1,0; 1,1]};
 a2 = {[0,1; 1,1]};
 b1 = {[1,1; 0,1]};
 b2 = {[1,1; 1,0]};
 sbL = {[1,0; 1,1]};
 sbR = {[1,1; 0,1]};
 cwL = {[1,0; 1,1]};
 cwR = {[1,1; 0,1]};
 birdL = {[0,1; 1,1]};
 birdR = {[1,1; 1,0]};
 dribL = {[0,1; 1,1]};
 dribR = {[1,1; 1,0]};
 hcsL = {[1,1; 0,1]};
 hcsR = {[0,1; 1,1]};
 aytL = {[1,1; 0,1]};
 aytR = {[0,1; 1,1]};
 keplerL = {[1,0; 1,1]};
 keplerR = {[0,1; 1,1]};
 fibQ = {[1,1; 1,0]};

gp_inline__location="../rationals.tex:463";
 sbLinv = sbL^-1;
 sbRinv = sbR^-1;

gp_inline__location="../rationals.tex:515";
 F_samples = {[0,1,1,2,3,5,8,13,21,34]};

gp_inline__location="../rationals.tex:536";
 V_samples = {[2,1,3,4,7,11,18,29,47,76]};

gp_inline__location="../rationals.tex:559";
 pq_is_acceptable(p,q) = {
   is_integer(p) && is_integer(p)
   && p>=1 && q>=1
   && gcd(p,q)==1;
 }

gp_inline__location="../rationals.tex:666";
 matrix_is_rationals_preserving(m) = {
   vecmin(m) >= 0
   && gcd(m[,1])==1
   && gcd(m[,2])==1
   && is_unimodular(m);
 }

gp_inline__location="../rationals.tex:1076";
 S = {[0,1;1,0]};

gp_inline__location="../rationals.tex:1114";
 \\ bit position of highest 1-bit
 tree_n_to_depth(n) = {
   n>=1 || error("tree_n_to_depth() is for n>=1");
   exponent(n);
 }

gp_inline__location="../rationals.tex:1126";
 \\ position 0 to 2^d-1 within row d = tree_n_to_depth(n)
 tree_n_to_row_offset(n) = n - 2^tree_n_to_depth(n);

gp_inline__location="../rationals.tex:1132";
 \\ depth d, row position n, matrices L,R
 \\ return 2x2 matrix
 tree_point_matrix(d,n, L,R, lowtohigh=0) = {
   d>=0 || error("tree_point_matrix() is for d>=0");
   (n>=0 && n < 2^d) || error("tree_point_matrix() is for 0 <= n < 2^d");
   my(v = binary_padded(n,d));
   if(lowtohigh,v=Vecrev(v));
   matid(2) * prod(i=1,#v, if(v[i],R,L));
 }

gp_inline__location="../rationals.tex:1142";
 \\ depth d, row position n, matrices L,R
 \\ return column matrix [p;q]
 tree_point(d,n, L,R, lowtohigh=0) = \
   tree_point_matrix(d,n, L,R, lowtohigh) * [1;1];

gp_inline__location="../rationals.tex:1147";
 \\ rows consecutively, so n most significant bit determines depth
 tree_point_n_matrix(n, L,R, lowtohigh=0) = {
   n>=1 || error("tree_point_n_matrix() is for n>=1");
   my(v=binary(n)[^1]);
   if(lowtohigh,v=Vecrev(v));
   prod(i=1,#v, if(v[i],R,L))*matid(2);
 }
 tree_point_n(n, L,R, lowtohigh=0) = \
   tree_point_n_matrix(n, L,R, lowtohigh) * [1;1];

gp_inline__location="../rationals.tex:1157";
 \\ depth d, row position n, matrices L,R
 \\ return 2x2 matrix
 tree_unpoint_bits(v, Linv,Rinv, lowtohigh=0) = {
   matsize(v) == [2,1] || error();
   my(limit=vec_to_p(v)+vec_to_q(v), l=List([]));
   while(v != [1;1],
     if(#l > limit, error("oops"));
     my(p=vec_to_p(v), q=vec_to_q(v));
     pq_is_acceptable(p,q) || error();
     if(p<q, v=Linv*v; listput(l,0),
             v=Rinv*v; listput(l,1)));
   if(lowtohigh, Vecrev(l), Vec(l));
 }

gp_inline__location="../rationals.tex:1175";
 tree_unpoint_n(v, Linv,Rinv, lowtohigh=0) = {
   fromdigits(concat(1,
                     tree_unpoint_bits(v, Linv,Rinv, lowtohigh)),
              2);
 }

gp_inline__location="../rationals.tex:1181";
 SB_point(d,n) = tree_point(d,n, sbL,sbR);
 SB_point_n(n) = tree_point_n(n, sbL,sbR);
 SB_unpoint_bits(v) = tree_unpoint_bits(v, sbLinv,sbRinv);
 SB_unpoint_n(v) = tree_unpoint_n(v, sbLinv,sbRinv);
 SB_row(d) = vector(2^d,n,n--; SB_point(d,n));
 SB_point_matrix(d,n) = tree_point_matrix(d,n, sbL, sbR);
 SB_point_n_matrix(n) = tree_point_n_matrix(n, sbL, sbR);
 SB_row_matrices(d) = vector(2^d,n,n--; SB_point_matrix(d,n));

gp_inline__location="../rationals.tex:1224";
 SB_p_new_highs_list(len) = {
   my(m=0,v=vector(len),upto=0);
   for(n=1,oo,
     if(upto>=#v,return(v));
     my(p=vec_to_p(SB_point_n(n)));
     if(p>m, v[upto++]=p; m=p));
 }

gp_inline__location="../rationals.tex:1245";
 SB_row_p_max(d) = F(d+2);

gp_inline__location="../rationals.tex:1297";
 \\ bit flip 0<->1 below the highest 1-bit = A054429
 complement_below_highest(n) = bitxor(n,bitneg(0,logint(n,2)));

gp_inline__location="../rationals.tex:1314";
 \\ A002487
 Diatomic(n) = {
   n>=0 || error();
   my(p=0, q=1);
   for(i=0,if(n,logint(n,2)), if(bittest(n,i),p+=q,q+=p));
   p;
 }

gp_inline__location="../rationals.tex:1338";
 \\ like full diatomic but p,q each mod 3
 \\ state machine is still 8 states when must distinguish 0,1,2
 \\ (as opposed to identifying 0 mod 3 can be 4 states)
 A071412(n) = {
   my(p=Mod(0,3),q=1);
   for(i=0,if(n,logint(n,2)), if(bittest(n,i),p+=q,q+=p));
   lift(p);
 }

gp_inline__location="../rationals.tex:1401";
 \\
 \\    1 +-----> 2 <-----+ 0
 \\      |       |       |          each state
 \\  0 +-1       |0,1    4-+ 1      is some destination of 0
 \\    +-^       v       ^-+
 \\      +------ 3 ------+
 \\           1      0
 Diatomic_is0mod3_state_table = {
   [ 1,2;    \\ 1 = start  0-reps
     3,3;    \\ 2 always down to 3
     4,1;    \\ 3 lower
     2,4 ];  \\ 4 right side 1-reps
 }
 Diatomic_is0mod3_by_states(n) = {
   my(state=1);
   for(i=0,if(n,logint(n,2)),
     state=Diatomic_is0mod3_state_table[state,bittest(n,i)+1]);
   state==1;
 }

gp_inline__location="../rationals.tex:1424";
 bit_reverse_k(n,k) = \
   my(v=Vecrev(binary(n))); fromdigits(Vec(v,k),2);

gp_inline__location="../rationals.tex:1433";
 Diatomic_is0mod3_state_count_gf(state) = \
   state_table_to_gfs(Diatomic_is0mod3_state_table,state)[1];

gp_inline__location="../rationals.tex:1446";
 Diatomic_is0mod3_state_count(state,k) = \
   gf_term(Diatomic_is0mod3_state_count_gf(state),k);
 /**/ Diatomic_is0mod3_state_count = memoize(Diatomic_is0mod3_state_count);

gp_inline__location="../rationals.tex:1451";
 \\ A122946 = Reznick count in 2^k <= n < 2^(k+1)
 A122946(n) = Diatomic_is0mod3_state_count(2,n);

gp_inline__location="../rationals.tex:1458";
 A089977(n) = {
   n>=0 || error();
   Diatomic_is0mod3_state_count(3,n+1);
 }

gp_inline__location="../rationals.tex:1488";
 A001607(n) = gf_term(x/(1 + x + 2*x^2), n);
 A110512(n) = gf_term((1+x)/(1 + x + 2*x^2), n);
 A169998(n) = gf_term((1+2*x)/(1 + x + 2*x^2), n);

gp_inline__location="../rationals.tex:1514";
 Diatomic_is0mod3_m = Mod(x, x^2+x+2);

gp_inline__location="../rationals.tex:1525";
 Diatomic_0mod3(n) = {
   my(k=0, debug=0);
   while(n>Diatomic_is0mod3_state_count(2,k), k++);
   k+=3;
   if(debug, print("find k="k));
   my(v=vector(k+1), state=1);
   while(k>=0,
     my(to=Diatomic_is0mod3_state_table[state,1],  \\ 0-bit
        count=Diatomic_is0mod3_state_count(to,k),  \\ its count
        bit=0);
     if(debug, print("  k="k" state="state"  compare n="n" vs to="to" count="count));
     if(n>=count,
        bit=1; n-=count;
           state=Diatomic_is0mod3_state_table[state,2],  \\ skip 0s
        bit=0; state=to);  \\ stay in 0s
     v[#v-k] = bit;
     if(debug, print("    bit="bit" pos="#v-k" remainder n="n));
     k--;
     );
   fromdigits(v,2);
 }

gp_inline__location="../rationals.tex:1580";
 Diatomic_is0mod3_state_count_m = Mod('x, 'x^2 + 'x + 2);
 Diatomic_is0mod3_state_count_coeffs = [1, 'x-1, -'x-1, 1];
 Diatomic_is0mod3_state_count_const = [2,0,0,-2];

gp_inline__location="../rationals.tex:1591";
 Diatomic_is0mod3_state_count_by_polmod(state,k) = {
   (polcoeff(lift(Diatomic_is0mod3_state_count_coeffs[state]
                 * Diatomic_is0mod3_state_count_m^k), 0)
    + 2^k
    + Diatomic_is0mod3_state_count_const[state] )/4;
 }

gp_inline__location="../rationals.tex:1601";
 Diatomic_0mod3_by_polmod(n) = {
   my(debug=0);
   my(k=if(n,logint(n,2)+3),
      p=Diatomic_is0mod3_state_count_m^k,
      state=1,
      ret=0);
   while(k>=0,
     my(t=Diatomic_is0mod3_state_table[state,1],
        c=(2^k 
         + Diatomic_is0mod3_state_count_const[t]
         + polcoeff(lift(p
            * Diatomic_is0mod3_state_count_coeffs[t]), 0)) /4);
     if(debug,print("k="k" state="state" n="n" vs c="c));
     if(n<c,
         if(debug,print("  bit 0"));
         state=t;
         ,
         if(debug,print("  bit 1"));
         n-=c;
         ret+=1<<k;
         state=Diatomic_is0mod3_state_table[state,2]);
     p/='x; k--);
   ret;
 }

gp_inline__location="../rationals.tex:1629";
 Diatomic_0mod3_by_polmod_four(n) = {
   my(debug=0,n_orig=n);
   if(n==0,return(0));
   n *= 4;
   my(k=if(n,logint(n,2))+15,
      p=Diatomic_is0mod3_state_count_m^k,
      state=1,
      ret=0);
   while(k>=0,
     my(t=Diatomic_is0mod3_state_table[state,1],
        c=(Diatomic_is0mod3_state_count_const[t]
                 + polcoeff(lift(p
                   * Diatomic_is0mod3_state_count_coeffs[t]), 0)));
     if(debug,print("k="k" state="state" n="n" vs c="c));
     \\n-c>=0 || print(n_orig" "n-c); \\ error();
     my(m=n-c);
     if(m < 2^k,
         if(debug,print("  bit 0"));
         state=t;
         ,
         if(debug,print("  bit 1"));
         \\ n = n-(c+2^k);
         n=m-2^k;
         ret+=1<<k;
         state=Diatomic_is0mod3_state_table[state,2]);
     p/='x; k--);
   ret;
 }

gp_inline__location="../rationals.tex:1662";
 { my(m=Mod('x, 'x^2+'x+2),
      f=[1,2, -'x-1,0, 1,-2, 'x-1,0],
      table=[1,3, 5,5, 7,1, 3,7]);
 Diatomic_0mod3_compact(n) =
   n<<=2;
   my(k=if(n,logint(n,2)+1), p=m^k, b=1<<k, S=1, ret=0);
   while(b,
     my(c=b+polcoeff(lift(f[S]*p),0)+f[S+1]);
     if(n>=c, n-=c; ret+=b; S++); S=table[S]; 
     p/='x; b>>=1);
   ret;
 }

gp_inline__location="../rationals.tex:1721";
 { my(m=Mod('x, 'x^2+'x+2),
      f=[1,2, -'x-1,0, 1,-2, 'x-1,0],
      table=[1,3, 5,5, 7,1, 3,7]);
 Diatomic_0mod3_by_bittest(n) =
   n<<=2; my(k=if(n,logint(n,2)+1), p=m^k, S=1);
   while(k>=0,
     my(t = n + (3<<k) - polcoeff(lift(f[S]*p),0) - f[S+1]);
     n%4==0 || error();
     t%4==0 || error();
     if(bittest(t,k+2), n=t; S++); S=table[S]; p/='x; k--);
   n>>2;
 }

gp_inline__location="../rationals.tex:1864";
 SB_p_samples = {[1,1,2,1,2,3,3,1,2,3,3,4,5,5,4]};

gp_inline__location="../rationals.tex:1875";
 \\ A007305 has extra initial 0,1, so root 1/1 at n=2
 \\ rather than n=1 as in SB_point_n().
 A007305(n) = if(n==0,0, n==1,1, vec_to_p(SB_point_n(n-1)));

gp_inline__location="../rationals.tex:1882";
 SB_q_samples = {[1,2,1,3,3,2,1,4,5,5,4,3,3,2,1]};

gp_inline__location="../rationals.tex:1893";
 \\ A047679 starts at 1/1 but OFFSET=0 so its n=0 is the
 \\ root rather than numbering n=1 root in SB_point_n().
 A047679(n) = vec_to_q(SB_point_n(n+1));

gp_inline__location="../rationals.tex:1974";
 \\ (and cf as num vertices at binomial tree height)
 CountHighOnes(n) = {   \\ A090996
   my(k=if(n,logint(n,2)));
   forstep(i=k,0,-1, bittest(n,i)||return(k-i));
   k+1;
 }

gp_inline__location="../rationals.tex:1985";
 CountHighOnes_samples = {[0,1,1,2,1,1,2,3,1,1,1,1,2,2,3]};

gp_inline__location="../rationals.tex:2012";
 r(d) = 5*2^(d-1) - 2;
 r_polygon(d) = concat(SB_row(d), Vecrev(SB_row(d+1)));

gp_inline__location="../rationals.tex:2018";
 r_samples = {[1/2,3,8,18,38,78,158,318,638]};

gp_inline__location="../rationals.tex:2022";
 r_samples2 = [1/2, 3, 8, 18, 38, 78, 158, 318, 638];

gp_inline__location="../rationals.tex:2045";
 R(d) = 5/2*(2^d-1) - 2*d;
 R_polygon(d) = concat(Vecrev(SB_row(d)), [ [1;1] ]);

gp_inline__location="../rationals.tex:2051";
 R_samples = {[0,(1)/(2),(7)/(2),(23)/(2),(59)/(2),(135)/(2),(291)/(2),(607)/(2)]};

gp_inline__location="../rationals.tex:2062";
 R_samples2 = [0, 1/2, 7/2, 23/2, 59/2, 135/2, 291/2, 607/2];

gp_inline__location="../rationals.tex:2190";
 r_between_polygon(d) =[[d+1;d+2], [d+2;d+1], [d+3;d+2], [d+2;d+3]];

gp_inline__location="../rationals.tex:2337";
 g(d) = gtotal(d) / r(d);

gp_inline__location="../rationals.tex:2339";
 g_samples = {[(4)/(3),2,3,(40)/(9),(125)/(19),(127)/(13),(1150)/(79),(3458)/(159)]};

gp_inline__location="../rationals.tex:2365";
 gtotal(d) = 19/6*3^d - d - 5/2;

gp_inline__location="../rationals.tex:2368";
 g_by_points(d) = {
   my(p=geom_centroid_of_polygon(r_polygon(d)),
      x=geom_point_x(p));
   x==geom_point_y(p) || error("oops");
   x;
 }

gp_inline__location="../rationals.tex:2377";
 gtotal_samples = {[2/3,6,24,80,250,762,2300,6916]};

gp_inline__location="../rationals.tex:2536";
  G(d) = if(d==0, 1, Gtotal(d) / R(d));

gp_inline__location="../rationals.tex:2539";
 G_samples = {[1,(4)/(3),(40)/(21),(8)/(3),(664)/(177),(2164)/(405),(6736)/(873),(20536)/(1821),(62032)/(3729)]};

gp_inline__location="../rationals.tex:2562";
 Gtotal(d) = (19*3^d - 6*d^2 - 24*d - 19) / 12;

gp_inline__location="../rationals.tex:2567";
 G_by_points(d) = {
   my(p=geom_centroid_of_polygon(R_polygon(d)),
      x=geom_point_x(p));
   x==geom_point_y(p) || error("oops");
   x;
 }

gp_inline__location="../rationals.tex:2576";
 Gtotal_samples = {[(2)/(3),(20)/(3),(92)/(3),(332)/(3),(1082)/(3),(3368)/(3),(10268)/(3),(31016)/(3),(93314)/(3)]};

gp_inline__location="../rationals.tex:2669";
 G_triangle_polygon = [1+I, 2+I, 1+2*I];

gp_inline__location="../rationals.tex:2675";
 G_rectangle_polygon = [1+2*I, 2+I, 'd+2+('d+1)*I, ('d+1)+('d+2)*I];
 G_rectangle_polygon_centroid = \
   geom_centroid_of_polygon(G_rectangle_polygon);

gp_inline__location="../rationals.tex:2727";
 Gtotal_d_plus_1_by_recurrence(d) = {
   3/2*G(d) * 2*R(d)
   + 1/2*(d+3) * 2*d
   + 4/3*1/2;
 }

gp_inline__location="../rationals.tex:2785";
 count_integer_points_between(v1,v2) = \
   my(d = v1-v2); max(0, gcd(vec_to_p(d), vec_to_q(d))-1);
 vector_count_integer_points_between(v) = \
   sum(i=1,#v-1, count_integer_points_between(v[i],v[i+1]));
 SB_row_count_integer_points_between(d) = \
   vector_count_integer_points_between(SB_row(d));

gp_inline__location="../rationals.tex:2882";
 non_coprime_c(d) = 2^d - 1;

gp_inline__location="../rationals.tex:2884";
 non_coprime_c_samples = {[0,1,3,7,15,31,63,127,255]};

gp_inline__location="../rationals.tex:2897";
 non_coprime_C(d) = 2^d - d - 1;
 non_coprime_C_from_sum(d) = sum(i=0,d-1,non_coprime_c(i));

gp_inline__location="../rationals.tex:2901";
 non_coprime_C_samples = {[0,0,1,4,11,26,57,120,247,502]};

gp_inline__location="../rationals.tex:2986";
 b(d) = 2^d + 2^(d+1);

gp_inline__location="../rationals.tex:2995";
 non_coprime_c_from_r(d) = r(d) - b(d)/2 + 1;

gp_inline__location="../rationals.tex:3005";
 B(d) = if(d==0, 1, 2^d + 2*d - 1);
 I_from_R(d) = R(d) - B(d)/2 + 1;
 non_coprime_C_from_R(d) = I_from_R(d) - sum(i=2,d-1, 2^i-2);

gp_inline__location="../rationals.tex:3054";
 vv_cross_product(v1,v2) = \
   vec_to_p(v1) * vec_to_q(v2) - vec_to_q(v1) * vec_to_p(v2);

gp_inline__location="../rationals.tex:3060";
 vvv_cross_product(v1,v2,v3) = vv_cross_product(v2-v1, v3-v2);
 vvv_to_turn_LSR(v1,v2,v3) = sign(vvv_cross_product(v1,v2,v3));

gp_inline__location="../rationals.tex:3067";
 tree_n_cross_product(n, L,R, lowtohigh=0) = {
   n>=2 || error("tree_n_cross_product() is for n>=2");
   my(v1=tree_point_n(n-1, L,R, lowtohigh),
      v2=tree_point_n(n,   L,R, lowtohigh),
      v3=tree_point_n(n+1, L,R, lowtohigh));
   vvv_cross_product(v1,v2,v3);
 }

gp_inline__location="../rationals.tex:3099";
 SB_turn_row1 = [-1, 1];             \\ R,L
 SB_turn_pattern = [1, -1, -1, 1];   \\ L,R,R,L
 SB_turn_n(n) = {
   n>=2 || error("SB_turn_n() is for n>=2");
   my(d=tree_n_to_depth(n),o=tree_n_to_row_offset(n));
   if(d==1, SB_turn_row1[o+1],
      o==0 || o==2^d-1, -1,
      SB_turn_pattern[(o%4)+1]);
 }

gp_inline__location="../rationals.tex:3437";
 CW_point(d,n) = tree_point(d,n, cwL, cwR, 1);
 CW_point_matrix(d,n) = tree_point_matrix(d,n, cwL,cwR, 1);
 CW_row(d) = vector(2^d,n,n--; CW_point(d,n));
 CW_row_matrices(d) = vector(2^d,n,n--; CW_point_matrix(d,n));

gp_inline__location="../rationals.tex:3442";
 CW_point_n(n) = tree_point_n(n, cwL, cwR, 1);
 CW_point_n_matrix(n) = tree_point_n_matrix(n, cwL,cwR, 1);
 CW_frac_n(n) = vec_to_frac(CW_point_n(n));

gp_inline__location="../rationals.tex:3572";
 CW_p_samples = {[1,1,2,1,3,2,3,1,4,3,5,2,5,3,4]};

gp_inline__location="../rationals.tex:3591";
 \\ this code per Charles Greathouse in A002487
 A002487(n) = \
   my(a=1,b=0); while(n>0, if(bitand(n,1), b+=a, a+=b); n>>=1); b;

gp_inline__location="../rationals.tex:3664";
 CountLowZeros(n) = valuation(n,2);

gp_inline__location="../rationals.tex:3666";
 CountLowOnes(n) = \
   my(count=0); while(n%2==1,n\=2;count++;); count;

gp_inline__location="../rationals.tex:3685";
 Knuth_x_iterate_to_n(xprev,n) = 1/(1 + 2*CountLowZeros(n) - xprev);

gp_inline__location="../rationals.tex:3699";
 CountLowZeros_samples = {[0,1,0,2,0,1,0,3,0,1,0,2]};

gp_inline__location="../rationals.tex:3850";
 CW_matrix_next(M) = {
   my(a=M[1,1],b=M[1,2],c=M[2,1],d=M[2,2]);
   if(c==0,return([1,0; b+1,1]));
   my(m,r); [m,r]=divrem(b,d);
   return([  c,           d;
           (2*m+1)*c - a, b+d-2*r]);
 }

gp_inline__location="../rationals.tex:3963";
 divrem_r_1_to_d(n,d) = {
   my(q,r); [q,r]=divrem(n,d);
   if(r==0,q--;r=d);
   n==q*d+r || error();
   [q,r];
 }

gp_inline__location="../rationals.tex:3998";
 CW_matrix_next_by_a_div_c(M) = {
   my(a=M[1,1],b=M[1,2],c=M[2,1],d=M[2,2]);
   if(c==0,return([1,0; b+1,1]));
   my(m,r); [m,r]=divrem_r_1_to_d(a,c);
   return([  c,        d;
           c+a-2*r,  (2*m+1)*d - b ]);
 }

gp_inline__location="../rationals.tex:4047";
 is_unimodular(m) = matdet(m)==1;
 pos_unimodulars(limit) = {
   my(l=List([]));
   forvec(v=vector(4,n,[0,limit]),
          my(m=[v[1],v[2]; v[3],v[4]]);
          if(is_unimodular(m)==1,listput(l,m)));
   Vec(l);
 }

gp_inline__location="../rationals.tex:4060";
 CW_tree_matrices(d) = {
   if(d==0, CW_row_matrices(d),
            concat(vector(d+1,d,d--; CW_row_matrices(d))));
 }

gp_inline__location="../rationals.tex:4066";
 pos_unimodulars_by_CW(limit) = {
   select(m->m[1,1]<=limit && m[1,2]<=limit &&
             m[2,1]<=limit && m[2,2]<=limit,
          CW_tree_matrices(limit));
 }

gp_inline__location="../rationals.tex:4077";
 all_unimodulars(limit) = {
   my(l=List([]));
   forvec(v=vector(4,n,[-limit,limit]),
          my(m=[v[1],v[2]; v[3],v[4]]);
          if(is_unimodular(m)==1,listput(l,m)));
   Vec(l);
 }

gp_inline__location="../rationals.tex:4104";
 neg_ad(m) = m[1,1]=-m[1,1]; m[2,2]=-m[2,2]; m;
 neg_ab(m) = m[1,1]=-m[1,1]; m[1,2]=-m[1,2]; m;

gp_inline__location="../rationals.tex:4126";
 Bird_point(d,n) = tree_point(d,n, birdL, birdR);
 Bird_row(d) = vector(2^d,n,n--; Bird_point(d,n));

gp_inline__location="../rationals.tex:4129";
 Bird_point_n(n) = tree_point_n(n, birdL, birdR);

gp_inline__location="../rationals.tex:4166";
 bird_l_picture = {[0,1;1,1]};

gp_inline__location="../rationals.tex:4182";
 bird_r_picture = {[1,1;1,0]};

gp_inline__location="../rationals.tex:4261";
 Bird_p_samples = {[1,1,2,2,1,3,3,3,3,1,2,5,4,4,5]};

gp_inline__location="../rationals.tex:4276";
 Bird_q_samples = {[1,2,1,3,3,1,2,5,4,4,5,2,1,3,3]};

gp_inline__location="../rationals.tex:4299";
 bird_l_calc = {[0,1;1,1]};

gp_inline__location="../rationals.tex:4308";
 bird_r_calc = {[1,1;1,0]};

gp_inline__location="../rationals.tex:4322";
 pq_first(L,R,d)       = L^d*[1;1];        /* 00000 */
 pq_last(L,R,d)        = R^d*[1;1];        /* 11111 */
 pq_second(L,R,d)      = L^(d-1)*R*[1;1];  /* 00001 */
 pq_second_last(L,R,d) = R^(d-1)*L*[1;1];  /* 11110 */
 pq_middle_prev(L,R,d)   = L*R^(d-2)*L*[1;1]; /* 01110 */
 pq_middle_first(L,R,d)  = L*R^(d-1)*[1;1];   /* 01111 */
 pq_middle_second(L,R,d) = R*L^(d-1)*[1;1];   /* 10000 */
 pq_middle_next(L,R,d)   = R*L^(d-2)*R*[1;1]; /* 10001 */

gp_inline__location="../rationals.tex:4331";
 BirdTurn_by_points(n) = {
   vvv_to_turn_LSR(Bird_point_n(n-1),
                   Bird_point_n(n),
                   Bird_point_n(n+1));
 }

gp_inline__location="../rationals.tex:4349";
 BirdTurn_initial = {[-1,1,1,1,1,0,0]};

gp_inline__location="../rationals.tex:4360";
 BirdTurn_formula(n) = {
   my(d=tree_n_to_depth(n));
   (-1)^(d - max(2,CountLows(n)));
 }
 BirdTurn(n) = {
   n>=2 || error("BirdTurn() is for n>=2");
   if(n<=8, BirdTurn_initial[n-1],
            BirdTurn_formula(n));
 }

gp_inline__location="../rationals.tex:4379";
 BirdTurn_samples = {[-1,1,1,1,1,0,0,-1,-1,-1,-1,-1,-1,1]};

gp_inline__location="../rationals.tex:4402";
 CountLows(n) = {
   my(d=tree_n_to_depth(n),r=tree_n_to_row_offset(n));
   my(v=binary_padded(r,d));
   forstep(i=#v-1,1,-1, if(v[i]!=v[#v],return(#v-i)));
   #v;
 }

gp_inline__location="../rationals.tex:4422";
 CountLows_samples = {[0,1,1,2,1,1,2,3,1,1,2,2,1,1,3,4]};

gp_inline__location="../rationals.tex:4452";
 A091090(n) = A007814(n+1) + 1 - A036987(n);
 A007814(n) = CountLowZeros(n);
 A036987(n) = is_power_of_2(n+1);

gp_inline__location="../rationals.tex:4626";
 Bird_last_cross_samples = {[2,0,5,8,26,63,170]};

gp_inline__location="../rationals.tex:4831";
 Drib_point(d,n) = tree_point(d,n, dribL, dribR, 1);
 Drib_row(d) = vector(2^d,n,n--; Drib_point(d,n));

gp_inline__location="../rationals.tex:4834";
 Drib_point_n(n) = tree_point_n(n, dribL, dribR, 1);

gp_inline__location="../rationals.tex:4948";
 Drib_p_samples = {[1,1,2,2,3,1,3,3,5,1,4,3,4,2,5]};

gp_inline__location="../rationals.tex:4961";
 A162911(n) = vec_to_p(Drib_point_n(n));

gp_inline__location="../rationals.tex:4966";
 Drib_q_samples = {[1,2,1,3,1,3,2,5,2,4,3,4,1,5,3]};

gp_inline__location="../rationals.tex:4979";
 A162912(n) = vec_to_q(Drib_point_n(n));

gp_inline__location="../rationals.tex:4988";
 lex_smallest_perm(len,from_func,to_func) = {
   my(upto=Map()); \\ want value => prev position in to_func()
   vector(len,n,
     my(want=from_func(n),
        i=0);
     mapisdefined(upto,want,&i);
     while(to_func(i++)!=want,);
     mapput(upto,want,i);
     i);
  }

gp_inline__location="../rationals.tex:5002";
 A334998_samps = {[
   1, 3, 2, 7, 4, 5, 6, 11, 8, 13, 10, 15, 12, 9, 14, 19, 16,
   29, 18, 23, 20, 17, 22, 27, 24, 21, 26, 31, 28, 25, 30, 35,
   32, 45, 34, 39, 36, 33, 38, 43, 40, 53, 42, 47, 44, 41, 46,
   51, 48, 61, 50, 55, 52, 49, 54, 59, 56, 37, 58, 63, 60, 57, 62]};
 A334999_samps = {[
   1,
   3, 2,
   5, 6, 7, 4,
   9, 14, 11, 8, 13, 10, 15, 12,
   17, 22, 19, 16, 21, 26, 23, 20, 25, 30, 27, 24, 29, 18, 31, 28,
   33, 38, 35, 32, 37, 58, 39, 36, 41, 46, 43, 40, 45, 34, 47, 44,
   49, 54, 51, 48, 53, 42, 55, 52, 57, 62, 59, 56, 61, 50, 63, 60
   ]};

gp_inline__location="../rationals.tex:5017";
 A334998(n) = {
   my(k=logint(n,2)-1,i=-1);
   while(i<k, i++;if(bittest(n,i)!=i%2,break));
   bitxor(n,bitneg(0,i+1));
 }
 A334999(n) = {
   my(v=binary(n));
   for(i=0,logint(n,2)-1,
     if(bittest(n,i)==!(i%2),
        n=bitxor(n,1<<i),
        n=bitxor(n,1<<i);break));
   n;
 }

gp_inline__location="../rationals.tex:5087";
 pos_non_alternating(n,low) = \
   for(i=0,oo, if(bittest(n,i)!=(i+low)%2, return(i)));

gp_inline__location="../rationals.tex:5109";
 A087230(n) = \
   my(k=-1); while(!bittest(n,k++) && bittest(n,k++),); k+1;

gp_inline__location="../rationals.tex:5139";
 A153141(n) = {
   my(hi=if(n,logint(n,2),0), lo=hi);
   while(lo>0 && bittest(n,lo--)==0, );
   bitxor(n, (1<<hi) - (1<<lo));
 }

gp_inline__location="../rationals.tex:5151";
 A153142(n) = {
   my(hi=if(n,logint(n,2),0), lo=hi);
   while(lo>0 && bittest(n,lo--)==1, );
   bitxor(n, (1<<hi) - (1<<lo));
 }

gp_inline__location="../rationals.tex:5183";
 \\ decrement with wrap-around in 2^k row, so high bit unchanged
 A153151(n) = if(n==0,0, if(logint(n,2)==valuation(n,2), n<<=1); n-1);

gp_inline__location="../rationals.tex:5194";
 \\ increment with wrap-around in 2^k row, so high bit unchanged
 A153152(n) = n++; if(logint(n,2)==valuation(n,2), n>>1, n);

gp_inline__location="../rationals.tex:5218";
 A154448(n) = {
   my(hi=if(n,logint(n,2),0), lo=hi, want=0);
   while(lo>0 && bittest(n,lo--)==want, want=!want);
   bitxor(n, (1<<hi) - (1<<lo));
 }

gp_inline__location="../rationals.tex:5228";
 A154447(n) = {
   my(hi=if(n,logint(n,2),0), lo=hi, want=1);
   while(lo>0 && bittest(n,lo--)==want, want=!want);
   bitxor(n, (1<<hi) - (1<<lo));
 }

gp_inline__location="../rationals.tex:5264";
 \\ swap adjacent pairs 2<->3
 A065190(n) = bitxor(n,n>1);

gp_inline__location="../rationals.tex:5291";
 HCS_point(d,n) = tree_point(d,n, hcsL, hcsR);
 HCS_point_n(n) = tree_point_n(n, hcsL, hcsR);
 HCS_row(d) = vector(2^d,n,n--; HCS_point(d,n));

gp_inline__location="../rationals.tex:5421";
 HCS_p_samples = {[1,2,1,3,3,1,2,4,5,4,5,1,2,3,3]};

gp_inline__location="../rationals.tex:5437";
 HCS_q_samples = {[1,1,2,1,2,3,3,1,2,3,3,4,5,4,5]};

gp_inline__location="../rationals.tex:5489";
 \\ dots  . 1 . 0 . 0 . 0 . 1 . 0 .
 \\ erase 0s, use 1s as separators into runs
 Czyz_Self_n_to_cfvec(n) = {
   n>=1 || error();
   my(v=binary(n),l=List([]),len=1);
   v=v[^1];  \\ drop high 1-bit
   for(i=1,#v,
     if(v[i]==1,listput(l,len);len=1, len++));
   listput(l,len);
   Vec(l);
 }
 Czyz_Self_cfvec_adjust(v) = {
   v[1]--;
   if(#v>1, v[#v]++);  \\ don't increase when #v==1
   v;
 }
 Czyz_Self_n_to_rat(n) = {
   n>=1 || error();
   contfrac_to_rational
     (Czyz_Self_cfvec_adjust(Czyz_Self_n_to_cfvec(n)));
 }
 \\ list page 369, indexed by natural numbers n, ie. n>=1
 Czyz_Self_samples = {[
                    0,
                    1, 
                 1/2, 2, 
             3/2, 1/3, 2/3, 3,
   5/2, 4/3, 5/3, 1/4, 2/5, 3/4, 3/5, 4
 ]};

gp_inline__location="../rationals.tex:5558";
 ThueMorse(n) = hammingweight(n) % 2;

gp_inline__location="../rationals.tex:5562";
 HCS_turn_n(n) = if(ThueMorse(n+1),-1,1);

gp_inline__location="../rationals.tex:5833";
 cross12(d) = dp1(d)*dq2(d) - dp2(d)*dq1(d);

gp_inline__location="../rationals.tex:5835";
 p1(d) = vec_to_p(pq_second_last(hcsL,hcsR,d));
 q1(d) = vec_to_q(pq_second_last(hcsL,hcsR,d));
 p2(d) = vec_to_p(pq_last(hcsL,hcsR,d));
 q2(d) = vec_to_q(pq_last(hcsL,hcsR,d));
 p3(d) = vec_to_p(pq_first(hcsL,hcsR,d+1));
 q3(d) = vec_to_q(pq_first(hcsL,hcsR,d+1));
 p4(d) = vec_to_p(pq_second(hcsL,hcsR,d+1));
 q4(d) = vec_to_q(pq_second(hcsL,hcsR,d+1));
 dp1(d) = p2(d) - p1(d);
 dq1(d) = q2(d) - q1(d);
 dp2(d) = p3(d) - p2(d);
 dq2(d) = q3(d) - q2(d);
 dp3(d) = p4(d) - p3(d);
 dq3(d) = q4(d) - q3(d);

gp_inline__location="../rationals.tex:5966";
 cross23(d) = dp2(d)*dq3(d) - dp3(d)*dq2(d);

gp_inline__location="../rationals.tex:6029";
 p1(d) = vec_to_p(pq_middle_prev(hcsL,hcsR,d));
 q1(d) = vec_to_q(pq_middle_prev(hcsL,hcsR,d));
 p2(d) = vec_to_p(pq_middle_first(hcsL,hcsR,d));
 q2(d) = vec_to_q(pq_middle_first(hcsL,hcsR,d));
 p3(d) = vec_to_p(pq_middle_second(hcsL,hcsR,d));
 q3(d) = vec_to_q(pq_middle_second(hcsL,hcsR,d));
 p4(d) = vec_to_p(pq_middle_next(hcsL,hcsR,d));
 q4(d) = vec_to_q(pq_middle_next(hcsL,hcsR,d));

gp_inline__location="../rationals.tex:6172";
 HCS_middle_first_cross_samples = {[2,1,7,8,22,34,70]};

gp_inline__location="../rationals.tex:6193";
 HCS_middle_second_cross_samples = {[-1,-5,-14,-34,-74,-152]};

gp_inline__location="../rationals.tex:6418";
 hcsRevL = a2;
 hcsRevR = b1;
 HCSrev_point(d,n) = tree_point(d,n, hcsRevL, hcsRevR);
 HCSrev_point_n(n) = tree_point_n(n, hcsRevL, hcsRevR);
 HCSrev_frac_n(n) = vec_to_frac(HCSrev_point_n(n));
 HCSrev_row(d) = vector(2^d,n,n--; HCSrev_point(d,n));

gp_inline__location="../rationals.tex:6433";
 HCSrev_p_samples = {[1,1,2,2,1,3,3,3,3,2,1,5,4,5,4]};

gp_inline__location="../rationals.tex:6448";
 HCSrev_q_samples = {[1,2,1,3,3,2,1,5,4,5,4,3,3,2,1]};

gp_inline__location="../rationals.tex:6468";
 AYT_point(d,n) = tree_point(d,n, aytL, aytR, 1);
 AYT_point_n(n) = tree_point_n(n, aytL, aytR, 1);
 AYT_frac_n(n) = vec_to_frac(AYT_point_n(n));

gp_inline__location="../rationals.tex:6472";
 AYT_row(d) = vector(2^d,n,n--; AYT_point(d,n));

gp_inline__location="../rationals.tex:6603";
 AYT_p_samples = {[1,2,1,3,1,3,2,4,1,4,3,5,2,5,3]};

gp_inline__location="../rationals.tex:6618";
 AYT_q_samples = {[1,1,2,1,3,2,3,1,4,3,4,2,5,3,5]};

gp_inline__location="../rationals.tex:6699";
 AYT_prev(v) = {
   my(p=vec_to_p(v),q=vec_to_q(v),m=p\q,r=p%q);
   [q*F(m-1) + r*F(m); q*F(m) + r*F(m+1)];
 }

gp_inline__location="../rationals.tex:6733";
 aytRevL = a2;
 aytRevR = b1;
 AYTrev_point(d,n) = tree_point(d,n, aytRevL, aytRevR, 1);
 AYTrev_point_n(n) = tree_point_n(n, aytRevL, aytRevR, 1);
 AYTrev_frac_n(n) = vec_to_frac(AYTrev_point_n(n));
 AYTrev_row(d) = vector(2^d,n,n--; AYTrev_point(d,n));

gp_inline__location="../rationals.tex:6749";
 AYTrev_p_samples = {[1,1,2,2,3,1,3,3,5,2,5,3,4,1,4]};

gp_inline__location="../rationals.tex:6764";
 AYTrev_q_samples = {[1,2,1,3,2,3,1,5,3,5,2,4,3,4,1]};

gp_inline__location="../rationals.tex:6784";
 Kepler_premult_C = [0,1;1,1];
 Kepler_point(d,n) = Kepler_premult_C * AYT_point(d,n);
 Kepler_row(d) = vector(2^d,n,n--; Kepler_point(d,n));

gp_inline__location="../rationals.tex:6788";
 Kepler_point_n_matrix(n) = \
   Kepler_premult_C * tree_point_n_matrix(n,aytL,aytR,1);
 Kepler_point_n(n) = Kepler_premult_C * AYT_point_n(n);

gp_inline__location="../rationals.tex:6901";
 Kepler_p_samples = {[1,1,2,1,3,2,3,1,4,3,4,2,5,3,5]};

gp_inline__location="../rationals.tex:6916";
 Kepler_q_samples = {[2,3,3,4,4,5,5,5,5,7,7,7,7,8,8]};

gp_inline__location="../rationals.tex:7007";
 bitrev(k,n) = fromdigits(Vecrev(binary_padded(n,k)),2);
 permute_by_bit_reversal(v,k) = \
   vector(#v,n,n--; v[1+bitrev(k,n)]);

gp_inline__location="../rationals.tex:7094";
 LR_matrix_product(v,L,R) = {
   if(#v==0,return(matid(matsize(L)[1])));
   prod(i=1,#v,if(v[i],L,R));
 }

gp_inline__location="../rationals.tex:7150";
 \\ Return just the highest 1-bit of n.
 high_bit(n) = if(n, 1<<logint(n,2));

gp_inline__location="../rationals.tex:7158";
 \\ Set another 1-bit immediately above the highest 1-bit of n.
 n_extra_high_1bit(n) = n + (high_bit(n)<<1);

gp_inline__location="../rationals.tex:7173";
 HCS_half_point(d,n) = tree_point(d+1,n+2^d, hcsL, hcsR);
 HCS_half_point_n(n) = HCS_point_n(n_extra_high_1bit(n));
 HCS_half_row(d) = vector(2^d,n,n--; HCS_half_point(d,n));

gp_inline__location="../rationals.tex:7232";
 HV(d,i) = [ F(i+1); F(i+2) + (d-i)*F(i+1) ];
 H_polygon(d) = {
   concat([ Vecrev(vector(d,i, HV(d,i))),         \\ down left
            [ [1;1] ],
            vector(d,i, point_transpose(HV(d,i))) \\ along right
          ]);
 }

gp_inline__location="../rationals.tex:7475";
 point_to_slope(a,b) = (vec_to_q(b) - vec_to_q(a)) / (vec_to_p(b) - vec_to_p(a));
 HV_slope_i_to_iplus1(d,i) = point_to_slope(HV(d,i+1), HV(d,i));

gp_inline__location="../rationals.tex:7507";
  HPO(d) = F(d+3)-1;
 HPO_samples = {[1,2,4,7,12,20,33,54,88,143,232]};

gp_inline__location="../rationals.tex:7613";
 hp(d) = if(d==0,1, F(d+3) + d-2);

gp_inline__location="../rationals.tex:7617";
 hp_samples = {[1,2,5,9,15,24,38,60,95]};

gp_inline__location="../rationals.tex:7621";
 hp_samples2 = [ 1, 2, 5,  9, 15, 24, 38, 60,  95, 151, 241 ];

gp_inline__location="../rationals.tex:7638";
 HP(d) = if(d==0,1, F(d+3) + 2*d - 2);

gp_inline__location="../rationals.tex:7642";
 HP_samples = {[1,3,7,12,19,29,44,67,103]};

gp_inline__location="../rationals.tex:7646";
 HP_samples2 = [ 1, 3, 7, 12, 19, 29, 44, 67, 103, 160, 251 ];

gp_inline__location="../rationals.tex:7749";
 H(d)     = 1/2 *   F(2*d+3) - d - 1;
 H_phi(d) = 1/2*F_phi(2*d+3);  /* the power only */

gp_inline__location="../rationals.tex:7755";
 H_samples = {[0,(1)/(2),(7)/(2),(26)/(2),(79)/(2),(221)/(2),(596)/(2),(1581)/(2),(4163)/(2)]};

gp_inline__location="../rationals.tex:7777";
 H_generated_samples = {[0,1/2,7/2,13,79/2,221/2,298,1581/2,4163/2,
   5463,28635/2,75001/2,98196,514201/2,1346239/2]};

gp_inline__location="../rationals.tex:7787";
 A027937_formula(n) = fibonacci(2*n+3) - 2*n - 2;
 A027937(n) = {
   n>=1 || error("A027937() is for n>=1");
   A027937_formula(n);
 }

gp_inline__location="../rationals.tex:7909";
 H_full(d) = F(d+2)^2 - F(d)^2/2 - sum(i=3,d,F(i)^2) - (d+1-3) - 5;

gp_inline__location="../rationals.tex:7913";
 H3(d) = F(d+2)^2 - F(d)^2/2 - F(d)*F(d+1) - d - 1;

gp_inline__location="../rationals.tex:8037";
 h(d) = F(2*d+3)/2 - d^2/2 - d - 1;

gp_inline__location="../rationals.tex:8041";
 h_samples = {[0,0,(3)/(2),(17)/(2),(63)/(2),(196)/(2),(560)/(2),(1532)/(2),(4099)/(2)]};

gp_inline__location="../rationals.tex:8056";
 h_generated_samples = {[0,0,3/2,17/2,63/2,98,280,766,4099/2,
   10845/2,28535/2,37440,98124,257016,1346043/2]};

gp_inline__location="../rationals.tex:8157";
 HG(d) = if(d==0, 1, HGtotal(d) / H(d));
 HG_phi(d) = HGtotal_phi(d) / H_phi(d);

gp_inline__location="../rationals.tex:8160";
 HG_samples = {[1,(4)/(3),(40)/(21),(109)/(39),(1004)/(237),(4372)/(663),(1557)/(149),(8818)/(527),(112152)/(4163),(237598)/(5463)]};

gp_inline__location="../rationals.tex:8189";
 HGtotal(d)     = 1/24 *   ( F(3*d+7) - 4*d*d - 22*d - 13 );
 HGtotal_phi(d) = 1/24 * F_phi(3*d+7);   /* the power only */

gp_inline__location="../rationals.tex:8197";
 HGtotal_samples = {[(2)/(3),(20)/(3),(109)/(3),(502)/(3),(2186)/(3),3114,13227,56076,237598,(3019648)/(3)]};

gp_inline__location="../rationals.tex:8214";
 HGtotal_samples2 = [ 2/3, 20/3, 109/3, 502/3, 2186/3, 3114, 13227, 56076, 237598, 3019648/3 ];

gp_inline__location="../rationals.tex:8247";
 HGtotal_square(d) = F(d+2)^2 * F(d+2)/2 ;

gp_inline__location="../rationals.tex:8254";
 HGtotal_top_middle(d) = F(d)^2/2 * (F(d+1)+F(d+2)+F(d+2))/3;

gp_inline__location="../rationals.tex:8268";
 HGtotal_side_triangle(d,i) = \
   F(i)^2*(0+0+F(i) + F(i+1) + F(i+2) + F(i+1)+(d-i+1)*F(i))/6;
 HGtotal_side_triangles(d) = sum(i=3,d, HGtotal_side_triangle(d,i));

gp_inline__location="../rationals.tex:8272";
 HGtotal_side_triangles_simplified(d) = \
 (2*F(3*d+5) + 3*F(3*d+4) - 24 * (-1)^d * F(d+1)- 30*d - 235) / 120;

gp_inline__location="../rationals.tex:8280";
 HGtotal_3todplus1(d) = (d+1-3)*(0+1+1+3+3+d+1)/6;

gp_inline__location="../rationals.tex:8288";
 HGtotal_0to3(d) = 5*11/10;

gp_inline__location="../rationals.tex:8292";
 HGtotal_by_parts(d) = {
   if(d==1, return(2/3));
   HGtotal_square(d) - HGtotal_top_middle(d) - HGtotal_side_triangles(d)
   - HGtotal_3todplus1(d) - HGtotal_0to3(d);
 }

gp_inline__location="../rationals.tex:8332";
 HG_over_F(d) = (HG(d)-1) / (F(d+2)-1);

gp_inline__location="../rationals.tex:8342";
 HG_over_F_phi(d) = {
   1/24 * phi^(3*d+7)/sqrt5
   / ( 1/2*phi^(2*d+3)/sqrt5 * phi^(d+2)/sqrt5 );
 }

gp_inline__location="../rationals.tex:8355";
 HG_over_F_limit = 1/24*(5 + 3*sqrt5);

gp_inline__location="../rationals.tex:8358";
 HG_over_F_limit_sample = {(048784183/100000000)};

gp_inline__location="../rationals.tex:8367";
 HG_over_F_limit_from_middle = 1/(42 + 18*sqrt5);

gp_inline__location="../rationals.tex:8371";
 HG_over_F_limit_from_middle_sample = {(0012158169/1000000000)};

gp_inline__location="../rationals.tex:8383";
 HG_over_diagonal(d) = (HG(d) - 1) / (F(d+2)-1/2*F(d)-1);

gp_inline__location="../rationals.tex:8392";
 HG_over_diagonal_phi(d) = {
   1/24 * phi^(3*d+7)/sqrt5
   / ( 1/2*phi^(2*d+3)/sqrt5
       * (phi^(d+2)/sqrt5 - 1/2*phi^d/sqrt5)
     );
 }

gp_inline__location="../rationals.tex:8403";
 HG_over_diagonal_limit = 1/12 * (5 + sqrt5);
 HG_over_diagonal_limit_sample = {(060300566/100000000)};

gp_inline__location="../rationals.tex:8420";
 HG_over_diagonal_limit_offset = 1/(165+75*sqrt5);

gp_inline__location="../rationals.tex:8423";
 HG_over_diagonal_limit_offset_sample = {(000300566/100000000)};

gp_inline__location="../rationals.tex:8574";
 A188516(n) = (fibonacci(n+3)-1)^2;

gp_inline__location="../rationals.tex:8580";
 \\  aaL  aaR
 \\   aL   aR
 \\    L    R
 A188516_is_permitted(aaL,aaR, aL,aR, L,R) = {
   if(aaL==1 && aL==1 && L==0, return(0));
   if(aaR==1 && aR==1 && R==0, return(0));
   1;
 }

gp_inline__location="../rationals.tex:8600";
 A188516_validate(elem) = {
   my(aL='none, aR='none, aaL='none, aaR='none, L,R);
   for(i=1,#elem,
     [L,R] = elem[i];
     A188516_is_permitted(aaL,aaR, aL,aR, L,R) || error();
     [aaL,aaR] = [aL,aR];
     [aL,aR] = [L,R]);
 }

gp_inline__location="../rationals.tex:8609";
 A188516_extend_list(vec) = {
   my(l=List([]));
   for(i=1,#vec,
     my(elem=vec[i],
        aL='none, aR='none, aaL='none, aaR='none);
     if(#elem>=1, [aL,aR] = elem[#elem]);
     if(#elem>=2, [aaL,aaR] = elem[#elem-1]);
     for(L=0,1,
       for(R=0,1,
         if(A188516_is_permitted(aaL,aaR, aL,aR, L,R),
           listput(l,concat(elem,[[L,R]]))))));
   Vec(l);
 }
 A188516_extend_list_with_validate(vec) = {
   for(i=1,#vec, A188516_validate(vec[i]));
   A188516_extend_list(vec);
 }
 A188516_list(len) = {
   my(vec=[ [] ]);  \\ one empty array
   for(i=1,len, vec=A188516_extend_list(vec));
   vec;
 }

gp_inline__location="../rationals.tex:8640";
 vector_has_110(v) = {
   for(i=3,#v, if(v[i-2]==1 && v[i-1]==1 && v[i]==0, return(1)));
   0;
 }

gp_inline__location="../rationals.tex:8702";
 AMR(d)     = (F(d+2) - 1)^2;
 AMR_phi(d) = F_phi(d+2)^2;  /* the power only */

gp_inline__location="../rationals.tex:8705";
 AMR_samples = {[0,1,4,16,49,144,400,1089,2916,7744,20449]};

gp_inline__location="../rationals.tex:8720";
 BMR(d) = 4*(F(d+2) - 1);

gp_inline__location="../rationals.tex:8722";
 BMR_samples = {[0,4,8,16,28,48,80,132,216,352,572]};

gp_inline__location="../rationals.tex:8979";
 w_num(s,d)= (F(d+2) - F(d-s+1))*s  + F(d-s+2)+s*F(d-s+1) - F(d);
 h_num(s,d)= (F(d+2) - 1)*s  + F(d+1) - 1;

gp_inline__location="../rationals.tex:8983";
 w_simp(s,d)= s*F(d+2) - F(d) + F(d-s+2);
 h_simp(s,d)= s*F(d+2) + F(d+1) - (s+1);

gp_inline__location="../rationals.tex:8991";
 rect_num(s,d) = w_num(s,d)*h_num(s,d);
 rect(s,d) = w_num(s,d)*h_num(s,d)/(s^2+1);

gp_inline__location="../rationals.tex:9000";
 diff(s,d) = rect(s,d) - AMR(d);

gp_inline__location="../rationals.tex:9059";
 dnum(s,d) = diff(s,d) * (s^2+1);

gp_inline__location="../rationals.tex:9108";
 incr_part1(s,d) = (F(d-1) - (s-1)*F(d-s))  * F(d+2);
 incr_part1a(s,d) = (F(s) - (s-1)) *F(d-s)  * F(d+2);
 incr_part1b(s,d) = F(s-1)         *F(d-s-1)* F(d+2);
 incr_part2(s,d) = 2*s*(F(d+2)-1) - 1;
 incr_part3(s,d) = F(d) + (-1)^(d+s)*F(s+2);
 incr_part4(s,d) = (s-1)*F(d-s);
 incr_part5(s,d) = F(d-s-2);
 incr(s,d) = {
     incr_part1a(s,d) + incr_part1b(s,d) + incr_part2(s,d)
   + incr_part3(s,d) + incr_part4(s,d) + incr_part5(s,d);
 }
 incr_from_dnum(s,d) = dnum(s+1,d) - dnum(s,d);

gp_inline__location="../rationals.tex:9193";
 AMR_over_H_phi(d) = ( phi^(2*d+4)/5 )/( 1/2*phi^(2*d+3)/sqrt5 );

gp_inline__location="../rationals.tex:9199";
 AMR_over_H_limit = 1 + 1/sqrt5;
 AMR_over_H_limit_sample = {(144721359/100000000)};

gp_inline__location="../rationals.tex:9247";
 rpp_by_sum(d) = vecsum(apply(v->vec_to_p(v)^2,SB_row(d)));

gp_inline__location="../rationals.tex:9258";
 rpp(d) = ([5,-2;1,0]^d * [5;1])[2,1];

gp_inline__location="../rationals.tex:9267";
 rpp_high_coeff = (1/2 + 5/34*sqrt17);
 rpp_by_powers(d) = {
     rpp_high_coeff      * (5/2 + 1/2*sqrt17)^d
   + (1/2 - 5/34*sqrt17) * (5/2 - 1/2*sqrt17)^d;

 }

gp_inline__location="../rationals.tex:9277";
 rpp_samples = {[1,5,23,105,479,2185,9967,45465]};

gp_inline__location="../rationals.tex:9290";
 rpq_by_sum(d) =vecsum(apply(v->vec_to_p(v)*vec_to_q(v),SB_row(d)));

gp_inline__location="../rationals.tex:9296";
 rpq(d) = ([5,-2;1,0]^d * [4;1])[2,1];

gp_inline__location="../rationals.tex:9305";
 rpq_high_coeff = (1/2 + 3/34*sqrt17);
 rpq_by_powers(d) = {
     rpq_high_coeff      * (5/2 + 1/2*sqrt17)^d
   + (1/2 - 3/34*sqrt17) * (5/2 - 1/2*sqrt17)^d;
 }

gp_inline__location="../rationals.tex:9314";
 rpq_samples = {[1,4,18,82,374,1706,7782,35498]};

gp_inline__location="../rationals.tex:9327";
 grpp(x) = 1/(1 - 5*x + 2*x^2);

gp_inline__location="../rationals.tex:9332";
 grpq(x) = (1-x)/(1 - 5*x + 2*x^2);

gp_inline__location="../rationals.tex:9419";
 ppow = 5/2 + 1/2*sqrt17;
 ppow_sample = {(4561552/1000000)};

gp_inline__location="../rationals.tex:9436";
 psmall = 5/2 - 1/2*sqrt17;
 psmall_sample = {(0438447/1000000)};

gp_inline__location="../rationals.tex:9449";
 \\ centroid of points in row d
 GProw_by_points(d) = geom_centroid_of_points(SB_row(d));
 GProw(d) = (3/2)^d;

gp_inline__location="../rationals.tex:9455";
 \\ return matrix of inertia of points of row d, about the centroid
 Irow_inertia_by_points(d) = {
   my(v=SB_row(d),
      g=geom_centroid_of_points(v));
   v=apply(z->z-g,v); \\ print(geom_centroid_of_points(v));
   geom_inertia_of_points(v);
 }

gp_inline__location="../rationals.tex:9556";
 Irow_inertia(d) = {
   my(x=Irow_x(d), y=Irow_y(d), xy=Irow_xy(d), z=Irow_z(d));
   [  x,   -xy, 0;
     -xy,   y,  0;
      0,    0,  z  ];
 }

gp_inline__location="../rationals.tex:9586";
 Irow_x(d) = rpp(d) - (9/2)^d;

gp_inline__location="../rationals.tex:9592";
 Irow_x_samples = {[0,(1)/(2),(11)/(4),(111)/(8),(1103)/(16),(10871)/(32)]};

gp_inline__location="../rationals.tex:9612";
 Irow_y(d) = Irow_x(d);

gp_inline__location="../rationals.tex:9620";
 Irow_z(d) = 2*Irow_x(d);

gp_inline__location="../rationals.tex:9626";
 Irow_xy(d) = rpq(d)  - (9/2)^d;

gp_inline__location="../rationals.tex:9632";
 Irow_xy_samples = {[0,-(1)/(2),-(9)/(4),-(73)/(8),-(577)/(16),-(4457)/(32)]};

gp_inline__location="../rationals.tex:9695";
 IrowP_inertia(d) = \
   geom_inertia_rotate_xydir(Irow_inertia(d), 1-I);

gp_inline__location="../rationals.tex:9717";
 IrowP_x(d) = 1/2*rpq(d+1) - 2*(9/2)^d;

gp_inline__location="../rationals.tex:9727";
 IrowP_x_samples = {[0,0,(1)/(2),(19)/(4),(263)/(8),(3207)/(16)]};

gp_inline__location="../rationals.tex:9756";
 IrowP_y(d) = rpp(d-1);

gp_inline__location="../rationals.tex:9765";
 IrowP_y_samples = {[0,1,5,23,105,479]};

gp_inline__location="../rationals.tex:9847";
 IrowP_ratio_limit = 4 + sqrt17;

gp_inline__location="../rationals.tex:9854";
 IrowP_ratio_sample = {(8123105/1000000)};

gp_inline__location="../rationals.tex:9877";
 \\ M is a 3x3 inertia matrix
 \\ return a matrix which has x coordinates sheared to
 \\ xnew = x + yf*y + zf*z
 inertia_xshear(M,yf,zf=0) = {
   my(Ix  = geom_inertia_Ix(M),
      Iy  = geom_inertia_Iy(M),
      Iz  = geom_inertia_Iz(M),
      Ixy = geom_inertia_Ixy(M),
      Ixz = geom_inertia_Ixz(M),
      Iyz = geom_inertia_Iyz(M),
      yy = (Ix+Iz-Iy)/2,
      zz = (Ix+Iy-Iz)/2,
      new_Ixy = Ixy + yf*yy + zf*Iyz,
      new_Ixz = Ixz + zf*zz + yf*Iyz,
      add = 2*yf*Ixy + 2*zf*Ixz + 2*yf*zf*Iyz + yf^2*yy + zf^2*zz,
      new_Iy = Iy + add,
      new_Iz = Iz + add);
   [ Ix,   -new_Ixy, -new_Ixz;
    -new_Ixy,  new_Iy,  -Iyz;
    -new_Ixz, -Iyz,   new_Iz ];
 }

gp_inline__location="../rationals.tex:9901";
 \\ M is a 3x3 inertia matrix
 \\ return a matrix which has x coordinates sheared to
 \\ xnew = x + y
 inertia_shear_xplusy(M) = {
   my(Ix  = geom_inertia_Ix(M),
      Iy  = geom_inertia_Iy(M),
      Iz  = geom_inertia_Iz(M),
      Ixy = geom_inertia_Ixy(M));
   geom_inertia_of_Ix_Iy_Ixy(Ix, Iz+2*Ixy, Ixy+Ix);
 }

gp_inline__location="../rationals.tex:9934";
 point_xshear(p, ys) = {
   my(x=geom_point_x(p),
      y=geom_point_y(p));
   [x + ys*y, y];
 }
 polygon_xshear(v, ys) = apply(p->point_xshear(p,ys), v);

gp_inline__location="../rationals.tex:9947";
 IOr_by_recurrence(d) = {
   if(d==0, return(geom_inertia_of_triangle([1,1],[2,1],[1,2])));
   my(M=IOr_by_recurrence(d-1));
   M=inertia_shear_xplusy(M);
   M + geom_inertia_transpose_xy(M)
   + geom_inertia_of_polygon(r_between_polygon(d-1));
 }

gp_inline__location="../rationals.tex:9988";
 IOrx(d) = {
   5/3*rpp(d) + 37/12*rpq(d)
   - d^2 - 3*d - 23/6;
 }

gp_inline__location="../rationals.tex:9995";
 IOrx_samples = {[(11)/(12),(77)/(6),80,406,(5759)/(3),8858,(121645)/(3)]};

gp_inline__location="../rationals.tex:10016";
 IOrxy(d) = 37/12*rpp(d) + 1/8*rpq(d) - 2*d - 7/3;

gp_inline__location="../rationals.tex:10020";
 IOrxy_samples = {[7/8,(139)/(12),(401)/(6),(977)/(3),(4540)/(3),6938,31690]};

gp_inline__location="../rationals.tex:10043";
 IOrP_inertia(d) = \
   geom_inertia_rotate_xydir(IOr_by_recurrence(d), 1-I);

gp_inline__location="../rationals.tex:10048";
 IOrP_x(d) = 19/4*rpp(d) + 77/24*rpq(d) - d^2 - 5*d - 37/6;

gp_inline__location="../rationals.tex:10052";
 IOrP_y(d) = -17/12*rpp(d) + 71/24*rpq(d) - d^2 - d - 3/2;

gp_inline__location="../rationals.tex:10384";
 \\ is_variable(str) returns true if str is the name of a variable,
 \\ meaning a "foo" not equal to its own symbol 'foo.
 is_variable(str) = eval(str)!=eval(Str("'",str));

gp_inline__location="../rationals.tex:10389";
 \\ exceptions b(d), h(d), r(d)
 char_variables_allowed = Set(["b","g","h","r"]);
 check_char_variables() = {
   my(char_variables_ok = 1);
   for(ch=Vecsmall("a")[1], Vecsmall("z")[1],
       my(str = Strchr(ch));
       if(is_variable(str) && ! setsearch(char_variables_allowed,str),
          print(str," = ",eval(str)); char_variables_ok = 0));
   char_variables_ok;
 }

kill(gp_inline__location);
