#!/usr/bin/perl -w

# Copyright 2014, 2017, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use Math::Libm 'hypot';
use Math::PlanePath::RationalsTree;
use Math::NumSeq::PlanePathTurn;
use Math::NumSeq::DigitSumModulo;

use FindBin;
use lib $FindBin::Bin;
use MyRationals 'xy_to_node_name';
use MyPrintwrap ':all';


my $depth = 4;

print "% generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::RationalsTree->new(tree_type => 'HCS');
my ($n_lo, $n_hi) = $path->tree_depth_to_n_range($depth);

my $turnseq = Math::NumSeq::PlanePathTurn->new(planepath_object => $path,
                                           turn_type => 'Left');

my $thuemorse = Math::NumSeq::DigitSumModulo->new (radix=>2, modulus=>2);

foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  my $turn = ($turnseq->ith($n) ? 'L' : 'R');
  my $n_bits = $n + 1;
  my $bits = ($thuemorse->ith($n_bits) ? 'O' : 'E');
  my $eol = ($n == $n_hi ? '' : " \\\\");
  printf "%6b & %s & %s%s\n", $n_bits, $bits, $turn, $eol;
}
print "  % end generated\n";

exit 0;
