#!/usr/bin/perl -w

# Copyright 2014, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz picture showing SB tree convex hull around a single row of
# points.

use 5.010;
use strict;
use List::Util 'max','min';
use Math::Geometry::Planar;
use Math::PlanePath::RationalsTree;

use FindBin;
use lib $FindBin::Bin;
use MyRationals;
use MyPrintwrap ':all';


my $depth = 5;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');

my $points = [];  # arrayref of arrayrefs [$x,$y]
my ($n_row, $n_end) = $path->tree_depth_to_n_range($depth);
foreach my $n ($n_row .. $n_end) {
  my ($x,$y) = $path->n_to_xy($n);
  push @$points, [$x,$y];
}

{
  print "  % row points\n";
  $Printwrap = 2;
  printwrap("\\fill");
  $Printwrap += 2;
  my $join = '';
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    my $end = ($p==$points->[-1] ? ';' : '');
    printwrap(" $join($x,$y)circle$end");
    $join = '--';
  }
  printwrap_newline();
}

my $planar = Math::Geometry::Planar->new;
$planar->points($points);
if (@$points > 3) { $planar = $planar->convexhull2; }
$points = $planar->points;

{
  $Printwrap = 2;
  printwrap("% hull boundary\n");
  printwrap("\\draw ");
  $Printwrap += 2;
  my $join = '';
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    printwrap("$join($x,$y)");
    $join = '--';
  }
  printwrap("--cycle;\n");
}

# printwrap("  % hull vertices\n");
# foreach my $p (@$points) {
#   my ($x,$y) = @$p;
#   printwrap("  \\fill ($x,$y) circle(.9ex);");
# }
# printwrap("--cycle;\n");
# 

# descent lines
# printwrap("  \\draw");
# foreach my $n ($path->n_start .. $path->tree_depth_to_n_end($depth-1)) {
#   my ($x,$y) = $path->n_to_xy($n);
#   foreach my $child_n ($path->tree_n_children($n)) {
#     my ($child_x,$child_y) = $path->n_to_xy($child_n);
#     printwrap("  ($x,$y)--($child_x,$child_y)");
#     $Printwrap = 2;
#   }
# }

print "  % end generated\n";
exit 0;
